/* -------------------------------------------------------------------------- */
#define INCL_DOSPROCESS       /* DOS process values (for DosZ) */
#define INCL_DOSEXCEPTIONS    /* DOS exception values */
#define INCL_ERRORS           /* DOS error values     */
#define INCL_DOSSEMAPHORES
#include <os2.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "./h/getopt.h"
#include "./h/lvm_intr.h"
#include "./h/usbcalls.h"
void RefreshLVM(void);
void writelog(char *text,char *pipeName);
BOOL term=FALSE,nosound=FALSE;
PCHAR logname;
ULONG _System MyTermHandler( PEXCEPTIONREPORTRECORD       p1,
                             PEXCEPTIONREGISTRATIONRECORD p2,
                             PCONTEXTRECORD               p3,
                             PVOID                        pv );

/* ------------------------------------------------------------------------- */
int main( int argc, char *argv[] )
{
 HEV hDevPluged;   // Event Set When an USB dev pluged in;
 HEV hDevUnpluged; // Event Set When an USB dev is Unpluged
 USBNOTIFY NotifyID; // Id Returned from UsbRegisterDeviceNotification
 INT opt,dsleep=600,sleep=dsleep;
 EXCEPTIONREGISTRATIONRECORD  RegRec  = {0};
 APIRET rc;
 USBHANDLE hUSBDevice;
 ULONG NumDevices;
 ULONG   ulPostCt;
 ULONG NumDevicesLast = 0;
 UCHAR ucBuffer[640];

/* Adding MyTermHandler */
RegRec.ExceptionHandler = (ERR)MyTermHandler;
rc = DosSetExceptionHandler( &RegRec );
if (rc != NO_ERROR) {
   fprintf(stderr,"ERROR: DosSetExceptionHandler error: return code = %u\n",rc);
   return 1;
}

// checking for driver presence
rc = UsbQueryNumberDevices(&NumDevices);
if (rc!=0) {
 fprintf(stderr,"ERROR: can`t query number devices, rc=%d",rc);
 return 1;
}

printf("USBMOUNTD version 0.1\n");
  while ((opt = getopt(argc, argv, "?nsl:t:")) != EOF)
    switch (opt)
    {
    case '?':
       printf ("\rUSAGE: usbmountd.exe [-n] [-l logname] [-t msec] [-s] [-?]\n");
       printf ("       -n      dont beep in speaker when device added\n");
       printf ("       -s      refresh LVM at startup\n");
       printf ("       -l      log filename (can be a pipe)\n");
       printf ("       -t      time to wait after device detection (default=%dms)\n",dsleep);
       printf ("       -?      show this help\n");
       return(0);
       break;
case 'n':
       nosound=TRUE;
       break;
    case 'l':
       logname=optarg;
       break;
    case 't':
       sleep=atol(optarg);
       break;
    case 's':
     if (NumDevices>0) RefreshLVM();
     break;
    }
rc=DosCreateEventSem("\\SEM32\\USBMOUNTD\\PLUGGED",&hDevPluged,
                     DCE_AUTORESET,FALSE);
if(rc) {fprintf(stderr,"ERROR: Can`t create event sem. Another copy is running ?\n");return(1);}
DosCreateEventSem("\\SEM32\\USBMOUNTD\\UNPLUGGED",&hDevUnpluged,DCE_AUTORESET,FALSE);
rc= UsbRegisterChangeNotification( &NotifyID, hDevPluged,hDevUnpluged);
writelog("USBMOUNTD started. Waitig for the event.\n",logname);
for (;;)
{
 DosWaitEventSem(hDevPluged,(ULONG) SEM_INDEFINITE_WAIT);
 if (term) break;
 DosSleep(sleep);
 RefreshLVM();
}
/* exiting */
rc=UsbDeregisterNotification(NotifyID);
DosCloseEventSem(hDevUnpluged);
DosCloseEventSem(hDevPluged);
return(0);
}

void RefreshLVM()
{
    CARDINAL32 Error_Code = 1;
    writelog("Refreshing LVM.\n",logname);
    if (nosound==FALSE) {
      DosBeep(300,100);
      DosBeep(100,100);
    }
    Rediscover_PRMs( &Error_Code );
    if ( Error_Code != LVM_ENGINE_NO_ERROR ) {
        fprintf(stderr,"ERROR: Rediscover_PRMs returned %d\n", Error_Code );
    }
    return;
}

ULONG _System MyTermHandler( PEXCEPTIONREPORTRECORD       p1,
                             PEXCEPTIONREGISTRATIONRECORD p2,
                             PCONTEXTRECORD               p3,
                             PVOID                        pv )
{
  APIRET   rc  = NO_ERROR;   /* Return code */
  if (p1->ExceptionNum == XCPT_SIGNAL) {
        rc = DosAcknowledgeSignalException ( XCPT_SIGNAL_BREAK );
        if (rc != NO_ERROR) {
          fprintf(stderr,"ERROR: DosAcknowledgeSignalException: return code = %u\n", rc);
          return 1;
        } else {
          printf("Terminating program\n");
          term=TRUE;
          return XCPT_CONTINUE_EXECUTION;  /* cont. execution */
        } /* endif */
  }
  return XCPT_CONTINUE_SEARCH;       /* Exception not resolved. */
}

 void writelog(char *text,char *pipeName)
 {
    CHAR buf[200];
    FILE *f;
    if (pipeName!=NULL)
    {
    sprintf(buf,"%.6s",pipeName);
    if (0 == strcmpi(buf,"\\PIPE\\")) f = fopen(pipeName,"w");
    else f = fopen(pipeName,"a");
    if(f)
    {
       fprintf(f,"%s",text);
       fclose(f);
    }
    }
    printf(text);
 }
